@@ -3,12 +3,12 @@
 //! Implements GF(2<sup>x</sup>) for various "natural" sizes
 //! such as 2<sup>8</sup>, 2<sup>16</sup> and 2<sup>32</sup>.
 //! 
-//! # Basic Use: doing maths in a particular field
+//! # Basic Use: doing maths on elements of a particular field
 //! 
-//! As a user, the steps to take to use this library are:
+//! The general outline for using this library is:
 //!
 //! * decide what "class" of field you want to use (GF(2<sup>8</sup>),
-//! GF(2<sup>16</sup>), etc.);
+//! GF(2<sup>16</sup>), …);
 //! 
 //! * decide if you want to use one of the optimised adaptations or
 //! are happy with the default, generic code;
@@ -22,29 +22,57 @@
 //! 
 //! 
 //! ```rust
-//! use guff::{GaloisField, F4};
+//! use guff::{GaloisField, F4, new_gf4};
 //! 
 //! fn main() {
-//!     // create a GF(2<sup>4</sup>) field from a struct
+//!     // Create a GF(2<sup>4</sup>) field from a struct
 //!     // * `19` is the field's irreducible polynomial
 //!     // * `3` is the same value with bit 0x10 removed
 //!     let f = guff::F4 { full : 19, compact : 3 };
 //!     
 //!     assert_eq!(f.pow(5,3), f.mul(5,f.mul(5,5)) );
+//! 
+//!     // The same, but using a constructor as syntactic sugar
+//!     let f2 = guff::new_gf4(19, 3);
+//!
+//!     assert_eq!(f2.pow(5,3), f2.mul(5,f2.mul(5,5)) );
+//!     assert_eq!(f.pow(5,3), f2.pow(5,3));
 //! }
 //! 
 //! ```
+//!
+
 //! 
+//! # Vector Operations
 //! 
+//! Many applications involving Galois Fields involve working with
+//! vectors of field elements instead of single elements. Various
+//! primitive methods are implemented:
 //! 
+//! * sum of vector elements
+//! * sum of two equal-length vectors
+//! * cross product (pairwise multiplication of elements)
+//! * dot product (sum of cross-product values)
+//! * scaling a vector by a constant
+//! * fused multiply add (scale and sum by a pair of constants across
+//!   vector)
 //! 
+//! These are all implemented using slices of the appropriate
+//! [ElementStore] type. Where necessary, if a vector (slice) type is
+//! to be returned, it must be handled by the user by passing in a
+//! mutable reference.  Also note that some operations (such as
+//! scaling a vector) are done in-place, so if the previous values
+//! need to be saved, they should be copied first.
 //! 
+//! See the [GaloisField] documentation for a full list of method
+//! signatures (prefixed with `vec_`).
 //! 
 //! # Crate Name
 //! 
 //! \* The crate name is deliberately hyperbolic:
 //!
-//! > Noun *guff* - unacceptable behavior (especially ludicrously false statements)
+//! > Noun *guff* - unacceptable behavior (especially ludicrously
+//!   false statements)
 
 use num_traits;
 use num::{PrimInt,One,Zero};
@@ -289,8 +317,78 @@ pub trait GaloisField {
 	    poly = poly >> 1;
 	}
     }
-	
-    
+
+    // Vector operations
+    fn vec_sum_elements(&self, v : &[Self::E]) -> Self::E {
+	let mut sum = Self::E::zero();
+	for e in v.iter() {
+            sum = sum ^ *e;
+	}
+	sum
+    }
+
+    fn vec_add_vec_in_place(&self,
+			    dest  : &mut [Self::E],
+			    other : &[Self::E] ) {
+	assert_eq!(dest.len(), other.len());
+	for (d,o) in dest.iter_mut().zip(other) {
+	    *d = *d ^ *o
+	}
+    }
+
+    fn vec_add_vecs_giving_other(&self,
+				 dest  : &mut [Self::E],
+				 a : &[Self::E],
+				 b : &[Self::E]) {
+	assert_eq!(dest.len(), a.len());
+	assert_eq!(a.len(), b.len());
+	let (mut a_iter, mut b_iter) = (a.iter(), b.iter());
+	for d in dest.iter_mut() {
+	    *d = *a_iter.next().unwrap() ^ *b_iter.next().unwrap()
+	}
+    }
+
+    fn vec_cross_product(&self,
+			    dest : &mut [Self::E],
+			    a : &[Self::E],
+			    b : &[Self::E] ) {
+	assert_eq!(dest.len(), a.len());
+	assert_eq!(a.len(), b.len());
+
+	let (mut a_iter, mut b_iter) = (a.iter(), b.iter());
+	for d in dest.iter_mut() {
+            *d = self.mul(*a_iter.next().unwrap(), *b_iter.next().unwrap())
+	}
+    }
+
+    fn vec_dot_product(&self, a : &[Self::E], b : &[Self::E]) -> Self::E
+    {
+	assert_eq!(a.len(), b.len());
+	let mut sum = Self::E::zero();
+	for (a_item, b_item) in a.iter().zip(b) {
+            sum = sum ^ self.mul(*a_item, *b_item);
+	}
+	sum
+    }
+
+    fn vec_constant_scale_in_place(&self,
+				   dest : &mut [Self::E],
+				   a    : Self::E) {
+	for d in dest.iter_mut() {
+            *d = self.mul(*d,  a)
+	}
+    }
+
+    fn vec_fma_in_place(&self,
+			dest : &mut [Self::E],
+			a    : Self::E,
+			b    : Self::E ) {
+	for d in dest.iter_mut() {
+            *d = self.mul(*d, a) ^ b;
+	}
+    }
+
+
     // Other accessors provide syntactic sugar
     
     /// Access Self::HIGH_BIT as a method
@@ -306,16 +404,16 @@ pub trait GaloisField {
 }
 
 
-/// A type implementing maths in GF(2<sup>4</sup>)
+/// A type implementing (default) maths in GF(2<sup>4</sup>)
 pub struct F4  { pub full : u8,  pub compact : u8 }
 
-/// A type implementing maths in GF(2<sup>8</sup>)
+/// A type implementing (default) maths in GF(2<sup>8</sup>)
 pub struct F8  { pub full : u16, pub compact : u8 }
 	
-/// A type implementing maths in GF(2<sup>16</sup>)
+/// A type implementing (default) maths in GF(2<sup>16</sup>)
 pub struct F16 { pub full : u32, pub compact : u16 }
 
-/// A type implementing maths in GF(2<sup>32</sup>)
+/// A type implementing (default) maths in GF(2<sup>32</sup>)
 pub struct F32 { pub full : u64, pub compact : u32 }
 
 impl GaloisField for F4 {
@@ -335,7 +433,8 @@ impl GaloisField for F4 {
 
 // Constructor for GF(2<sup>4</sup>)
 #[allow(dead_code)]
-/// Create a new  GF(2<sup>4</sup>) field from a given field polynomial
+/// Create a new GF(2<sup>4</sup>) field with a supplied field
+/// polynomial (using the default implementation)
 pub fn new_gf4(full : u8, compact : u8) -> F4  {
     F4 {full : full, compact : compact}
 }
@@ -357,7 +456,8 @@ impl GaloisField for F8 {
 
 // Constructor for GF(2<sup>8</sup>)
 #[allow(dead_code)]
-/// Create a new  GF(2<sup>8</sup>) field from a given field polynomial
+/// Create a new GF(2<sup>8</sup>) field with a supplied field
+/// polynomial (using the default implementation)
 pub fn new_gf8(full : u16, compact : u8) -> F8  {
     F8 {full : full, compact : compact}
 }
@@ -379,7 +479,8 @@ impl GaloisField for F16 {
 
 // Constructor for GF(2<sup>16</sup>)
 #[allow(dead_code)]
-/// Create a new  GF(2<sup>16</sup>) field from a given field polynomial
+/// Create a new GF(2<sup>16</sup>) field with a supplied field
+/// polynomial (using the default implementation)
 pub fn new_gf16(full : u32, compact : u16) -> F16  {
     F16 {full : full, compact : compact}
 }
@@ -401,7 +502,8 @@ impl GaloisField for F32 {
 
 // Constructor for GF(2<sup>32</sup>)
 #[allow(dead_code)]
-/// Create a new  GF(2<sup>32</sup>) field from a given field polynomial
+/// Create a new GF(2<sup>32</sup>) field with a supplied field
+/// polynomial (using the default implementation)
 pub fn new_gf32(full : u64, compact : u32) -> F32  {
     F32 {full : full, compact : compact}
 }
